const request = require('request');
const AsksuiteProcessingNaturalUtils = require('../AsksuiteProcessingNaturalUtils');
const { ErrorHandlerService } = require('./ErrorHandlerService');

class PrometheusAcessor {
  constructor(config) {
    this.PROMETHEUS_URL = config.PROMETHEUS_URL;
    this.PROMETHEUS_LAYER_PARAMS = config.PROMETHEUS_LAYER_PARAMS;
    this.utils = new AsksuiteProcessingNaturalUtils();
  }

  resolveText(request) {
    return new Promise(resolve => {
      const executor = async () => {
        let response = null;
        let dialog = null;
        try {
          response = await this.requestData(this.PROMETHEUS_URL, {
            sentence: request.text,
            companyId: request.companyId,
            language: request.language,
            ...this.PROMETHEUS_LAYER_PARAMS,
          });

          const data = JSON.parse(response.body);
          const intent = data.intentResult.intent;
          if (!intent) {
            throw { message: response.statusMessage, code: response.statusCode };
          }

          dialog = this.utils.findDialogByIntent(request, intent);
          dialog.processingStep = {
            ...dialog.processingStep,
            text: request.text,
            language: request.language,
            resolver: 'PROMETHEUS',
            ...data,
          };
        } catch (e) {
          const errorData = ErrorHandlerService.accessorErrorHandler(e);
          dialog = {
            processingStep: {
              fallbackReason: 'intent_not_found',
              intentFound: null,
              text: request.text,
              language: request.language,
              resolver: 'PROMETHEUS',
              errorData,
            },
          };
          console.log('[PrometheusAccessor error]', e);
        }
        resolve(dialog);
      };

      executor();
    });
  }

  requestData(url, params) {
    return new Promise((resolve, reject) => {
      request.get(url, { qs: params, timeout: 20 * 1000 }, (err, res) => {
        if (err) {
          reject(err);
        }
        resolve(res);
      });
    });
  }
}

module.exports = PrometheusAcessor;
